/*
	ncbricks - ncurses based brick breaker
	Copyright (C) 2017-2018  Aron Barath

	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
	02110-1301  USA
*/

#ifndef VEC2_H
#define VEC2_H

struct vec2
{
	int x;
	int y;
};

struct rect2
{
	struct vec2 pos;
	struct vec2 end;
};

int vec2_length(const struct vec2* const vec);
int vec2_distance(const struct vec2* const p1, const struct vec2* const p2);
int vec2_L1norm(const struct vec2* const p1, const struct vec2* const p2);
void vec2_smul(struct vec2* const dst, const struct vec2* const vec, int scalar);
void vec2_sdiv(struct vec2* const dst, const struct vec2* const vec, int scalar);
void vec2_smuldiv(struct vec2* const dst, const struct vec2* const vec, int s_mul, int s_div);
void vec2_add(struct vec2* const dst, const struct vec2* const v1, const struct vec2* const v2);
void vec2_sub(struct vec2* const dst, const struct vec2* const v1, const struct vec2* const v2);
void vec2_random(struct vec2* const dst, const int magnitude);
int vec2_intersect(struct vec2* const point, const struct vec2* const line1s, const struct vec2* const line1e, const struct vec2* const line2s, const struct vec2* const line2e);
int vec2_intersect_vr(struct vec2* const point, const struct vec2* const line1s, const struct vec2* const line1e, const int line2sx, const int line2sy, const int line2ex, const int line2ey); /* line 1 in vector form; line 2 in raw form */
void rect2_normalize(struct rect2* const rect);
int rect2_contains(const struct rect2* const rect, const struct vec2* const point);

#endif /* VEC2_H */
